%% 
%   you do not need to run the whole script
%   remember you can use ctrl+Enter to execute one block at a time
%% PARAMETERS

K = 100; % size of subsampled shape
mdsDim = 2; % number of dimension to embed in

%% PREPARATION

% load meshes
[M1.VERT, M1.TRIV] = read_off('shapes/lion-03_v1002.off');
[M2.VERT, M2.TRIV] = read_off('shapes/lion-09_v1002.off');

M1.n = size(M1.VERT,1);
M2.n = size(M2.VERT,1);

% precalculated distance matrix (the ordering of indices is compatiblen in
% the example shapes)
load('shapes/lions_distances.mat')

numberOfVertices1 = size(M1.VERT,1);
numberOfVertices2 = size(M2.VERT,1);

%% FPS

figure
subplot(1,2,1);
v1 = randi(numberOfVertices1);
S1 = metricfps(K, v1, D1);
f1 = metricvoronoi(S1,D1);
meshplot(M1,f1);
hold on
scatter3(M1.VERT(S1,1),M1.VERT(S1,2),M1.VERT(S1,3),25,[1 1 1],'o','filled');
hold off

subplot(1,2,2);
v2 = randi(numberOfVertices2);
S2 = metricfps(K, v2, D2);
f2 = metricvoronoi(S2,D2);
meshplot(M2,f2);
hold on
scatter3(M2.VERT(S2,1),M2.VERT(S2,2),M2.VERT(S2,3),25,[1 1 1],'o','filled');
hold off

%% MDS

%metrics restricted to the FPS
D1S = D1(S1,S1);
D2S = D2(S2,S2);

n = size(D1S,1);
assert((n==size(D1S,2))&&(n==size(D2S,1))&&(n==size(D2S,2)));


[Z1, ~] = mds(D1S, mdsDim, 2e-4, 1e-5, 10000);
assert(isequal(size(Z1),[n,mdsDim]), 'Point matrix Z1 has wrong dimensions');
[Z2, ~] = mds(D2S, mdsDim, 2e-4, 1e-5, 10000);
assert(isequal(size(Z2),[n,mdsDim]), 'Point matrix Z2 has wrong dimensions');

[Zhat1, Zhat2] = alignpoints(Z1, Z2);
assert(isequal(size(Zhat1),[n,mdsDim]), 'Point matrix Zhat1 has wrong dimensions');
assert(isequal(size(Zhat2),[n,mdsDim]), 'Point matrix Zhat2 has wrong dimensions');

%% VISUALIZE EMBEDDINGS AND ALIGN AXES

%the aligned points can be mirrored along some axis
%experiment with values of +1 or -1 to account for this
dimSigns = [1 -1 1];
if (mdsDim==2)
    figure();
    plot(Zhat1(:,1),Zhat1(:,2),'r.'); axis off, axis equal, drawnow
    hold on
    plot(dimSigns(1)*Zhat2(:,1),dimSigns(2)*Zhat2(:,2),'b.'); axis off, axis equal, drawnow
    hold off;
end
if (mdsDim==3)
    figure();
    plot3(Zhat1(:,1),Zhat1(:,2),Zhat1(:,3),'r.'); axis off, axis equal, drawnow
    hold on
    plot3(dimSigns(1)*Zhat2(:,1),dimSigns(2)*Zhat2(:,2),dimSigns(3)*Zhat2(:,3),'b.'); axis off, axis equal, drawnow
    hold off;
end

%%

Zhat2(:,1) = dimSigns(1)*Zhat2(:,1);
Zhat2(:,2) = dimSigns(2)*Zhat2(:,2);
if(mdsDim == 3)
   Zhat2(:,3) = dimSigns(3)*Zhat2(:,3);
end

%% COMPUTE MATCHING FROM MDS

C = extract_matching(Zhat1, Zhat2);

C = [S1, S2(C(:,2))];

rgb_coloring(M1, M2, C);

%%
% special thanks to Thomas Windheuser for most parts of the code